/*
 *  gridlock: A ncurses application to calculate the Maidenhead Grid
 *  Locator of a given position or vice-versa. Also to calculate
 *  bearing and distance to a remote position.
 *
 *  Copyright (C) 2001  Neoklis Kyriazis
 *
 *  This program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */


/*  main.c
 *  
 *  Main body of gridloc application
 */  

#include "gridloc.h"

FILE
  *marker_fd,   /* Marker file for xplanet with sat/obs location */
  *greatarc_fd; /* Great circle arc co-ordinates for xplanet     */

/* Name strings for above files */
char
  markerfile[50],
  greatarcfile[50];

/* Signal handler */
static void Signal_Handler( int signal );

int
main( void )
{
  /* New and old actions for sigaction() */
  struct sigaction sa_new, sa_old;

  /* Initialize new actions */
  sa_new.sa_handler = Signal_Handler;
  sigemptyset( &sa_new.sa_mask );
  sa_new.sa_flags = 0;

  /* Register function to handle signals */
  sigaction( SIGINT,  &sa_new, &sa_old );
  sigaction( SIGSEGV, &sa_new, 0 );
  sigaction( SIGFPE,  &sa_new, 0 );
  sigaction( SIGTERM, &sa_new, 0 );
  sigaction( SIGABRT, &sa_new, 0 );

  /* Initialize ncurses */
  initscr();
  keypad( stdscr, TRUE );
  nonl();
  cbreak();
  noecho();

  /* Assuming terminal has color! */
  if( has_colors() )
    {
      start_color();

      init_pair( COLOR_WHITE,     COLOR_WHITE,  COLOR_BLACK  );
      init_pair( COLOR_RED,	  COLOR_RED,    COLOR_BLACK  );
      init_pair( COLOR_GREEN,	  COLOR_GREEN,  COLOR_BLACK  );
      init_pair( COLOR_BLUE,	  COLOR_BLUE,   COLOR_BLACK  );
      init_pair( COLOR_YELLOW,	  COLOR_YELLOW, COLOR_BLACK  );
      init_pair( COLOR_CYAN,	  COLOR_CYAN,   COLOR_BLACK  );
      init_pair( COLOR_MAGENTA,	  COLOR_MAGENTA,COLOR_BLACK  );
      init_pair( COLOR_KEYSTROKE, COLOR_YELLOW, COLOR_BLUE   );
      init_pair( COLOR_COMMAND,   COLOR_WHITE,  COLOR_BLUE   );
      init_pair( COLOR_ERROR,     COLOR_WHITE,  COLOR_RED    );
      init_pair( COLOR_APPROX,    COLOR_WHITE,  COLOR_YELLOW );
      init_pair( COLOR_FINISH,    COLOR_WHITE,  COLOR_GREEN  );
    } 

  /* Go to the ncurses gui */
  Post_Gridloc_Form();

  Signal_Handler( 0 );

  return(0);

} /* End of 'main()' */

/*------------------------------------------------------------------------*/


/*  Display_Location
 *
 *  Displays satellite tracks using xearth or xplanet
 */

int
Display_Location( char *grid_loc,
                  double home_lon, double home_lat,
		  double remt_lon, double remt_lat,
		  double remt_bng, double remt_dst )
{
  int error;  /* Error flag for system calls */

  char command[255]; /* xplanet command string */

  error = 0;

  /* Prepare xplanet's file names */
  sprintf( markerfile,   "%s/.markerfile",   getenv("HOME") );
  sprintf( greatarcfile, "%s/.greatarcfile", getenv("HOME") );

  /* Open xplanet file descriptors */
  marker_fd   = fopen( markerfile,   "w" );
  greatarc_fd = fopen( greatarcfile, "w" );

  /* Enter "Home" position in marker file */
  fprintf( marker_fd, "%8.3f %8.3f \"%s %03d/%02d\" %s\n",
	   home_lat, home_lon, HOME_NAME,
	   (int)home_lon, (int)home_lat, "color=red" );

  /* Enter "Remote" position in marker file */
  fprintf( marker_fd, "%8.3f %8.3f \"%s %03d/%d\" %s\n",
	   remt_lat, remt_lon, grid_loc,
	   (int)remt_bng, (int)remt_dst, "color=yellow" );

  /* Enter Home/Remote positions in greatarc file */
  fprintf( greatarc_fd, "%8.3f %8.3f %8.3f %8.3f %s\n",
	   home_lat, home_lon, remt_lat, remt_lon, "color=yellow" );

  fclose( greatarc_fd );
  fclose( marker_fd );

  /* Prepare command string for xplanet */
  sprintf( command, "xplanet -markerfile %s -greatarcfile %s",
	   markerfile, greatarcfile );

  /* Invoke xplanet */
  error = system( command );

  /* On error delete xplanet's files */
  if( error )
    {
      unlink( greatarcfile );
      unlink( markerfile );
    }

  return( error );

} /* End of Display_Location() */

/*------------------------------------------------------------------------*/
 
/*  Sig_Handler()
 *
 *  Signal Action Handler function
 */

static void Signal_Handler( int signal )
{
  endwin();

  /* Delete xplanet files */
  unlink( markerfile );
  unlink( greatarcfile );

  switch( signal )
    {
    case 0       : exit(0);
    case SIGINT  : puts( "gridloc: Exiting via User Interrupt" );
      break;
    case SIGSEGV : puts( "gridloc: Segmentation Fault" );
      break;
    case SIGFPE  : puts( "gridloc: Floating Point Exception" );
      break;
    case SIGABRT : puts( "gridloc: Abort Signal received" );
      break;
    case SIGTERM : puts( "gridlock: Termination Request received" );
    }

  exit( -1 );

} /* End of Sig_Handler() */

/*------------------------------------------------------------------------*/

/* Functions for testing and setting/clearing flags */

/* An int variable holding the single-bit flags */
static int Flags = 0;

int
isFlagSet(int flag)
{
  return (Flags & flag);
}

int
isFlagClear(int flag)
{
  return (~Flags & flag);
}

void
SetFlag(int flag)
{
  Flags |= flag;
}

void
ClearFlag(int flag)
{
  Flags &= ~flag;
}

void
ToggleFlag(int flag)
{
  Flags ^= flag;
}

/*------------------------------------------------------------------*/

